# ----------------------------------------------------------------------------
# Copyright (c) 2022--, Panpiper development team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file COPYING.txt, distributed with this software.
# ----------------------------------------------------------------------------

import sys
import yaml
import re
import os
import subprocess
import logging
import glob
import shlex
import subprocess


class Workflow(object):

    def __init__(self, obj):
        for key, val in vars(obj).items():
            setattr(self, key, val)

    def run(self, snakefile):
        """
        Construct the snakemake command based off user-defined parameters
        Will produce a command, cmd, which will be launched

        Parameters 
        ----------
        self: object, required
        Contains the parameters defined by the user

        snakefile: string, required
        The snakefile selected for the desired operation
        """
        # Define core snakemake command
        # need to edit j
        cmd = ['snakemake',
               '--use-conda',
               '--latency-wait', '20',
               '--rerun-incomplete',
               '--jobs',str(self.max_jobs),
               '-s', snakefile,
               '--config',
               'out='+self.output,
               'fastq='+self.fastq,
               'fasta='+self.fasta,
               'list='+self.sample_list,
               'ref='+self.reference,
               'structure='+self.structure,
               'unitig='+self.unitig,
               'tree='+self.tree,
               'genes='+self.genes,
               'pheno='+self.pheno,
               'scripts='+self.scripts_dir,
               'params='+self.params]

        # If run on cluster
        if self.cluster:
            cmd += ['--profile', self.profile]

        # Only install conda envs if only_conda
        if self.only_conda:
            logging.info('Only creating conda environments.')
            cmd.append(' --conda-create-envs-only')

        if self.dry_run:
            logging.info('Creating a dry run.')
            cmd.append('-n')
        # If unlocking

        if self.unlock:
            logging.info('Unlocking working directory.')
            cmd.append('--unlock')

        # make log directory
        try:
            os.mkdir(self.output + 'report')
        except FileExistsError:
            pass

        print(cmd)
        logging.info(' '.join(cmd))
        args = shlex.split(' '.join(cmd))

        # Run the snakemake workflow
        process = subprocess.Popen(
            args, stderr=subprocess.STDOUT, stdout=subprocess.PIPE, text=True)
        if self.log_lvl == 'DEBUG':
            for line in iter(process.stdout.readline, ""):
                logging.debug(line.strip())

        process.wait()
        logging.debug('Snakemake returncode: '+str(process.returncode))

        # Check returncode
        if process.returncode != 0:
            for line in iter(process.stdout.readline, ""):
                logging.error(line.strip())
            logging.error('Snakemake returncode: '+str(process.returncode))
            sys.exit()

        # If unlocking exit program
        if self.unlock:
            sys.exit()



#!/usr/bin/env python

# -----------------------------------------------------------------------------
# Copyright (c) 2011-2017, The BIOM Format Development Team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file COPYING.txt, distributed with this software.
# -----------------------------------------------------------------------------

from os.path import abspath, dirname, join
import tempfile

import numpy as np

from biom.cli.table_converter import _convert
from biom.parse import MetadataMap
from biom.table import Table
from biom import load_table
from biom.parse import biom_open, parse_biom_table
from unittest import TestCase, main


class TableConverterTests(TestCase):

    def setUp(self):
        """Set up data for use in unit tests."""
        self.cmd = _convert
        self.output_filepath = tempfile.NamedTemporaryFile().name

        with tempfile.NamedTemporaryFile('w') as fh:
            fh.write(biom1)
            fh.flush()
            self.biom_table1 = load_table(fh.name)

        self.biom_lines1 = biom1.split('\n')
        with tempfile.NamedTemporaryFile('w') as fh:
            fh.write(classic1)
            fh.flush()
            self.classic_biom1 = load_table(fh.name)

        self.sample_md1 = MetadataMap.from_file(sample_md1.split('\n'))

        test_data_dir = join(dirname(abspath(__file__)), 'test_data')
        self.json_collapsed_obs = join(test_data_dir,
                                       'json_obs_collapsed.biom')
        self.json_collapsed_samples = join(test_data_dir,
                                           'json_sample_collapsed.biom')

    def test_classic_to_biom(self):
        """Correctly converts classic to biom."""
        self.cmd(table=self.classic_biom1,
                 output_filepath=self.output_filepath,
                 to_json=True, table_type='OTU table')

        obs = load_table(self.output_filepath)
        self.assertEqual(type(obs), Table)
        self.assertEqual(len(obs.ids()), 9)
        self.assertEqual(len(obs.ids(axis='observation')), 14)
        self.assertEqual(obs.metadata(), None)
        self.assertNotEqual(obs.metadata(axis='observation'), None)

    def test_classic_to_biom_with_metadata(self):
        """Correctly converts classic to biom with metadata."""
        # No processing of metadata.
        obs = self.cmd(table=self.classic_biom1,
                       output_filepath=self.output_filepath,
                       sample_metadata=self.sample_md1, to_json=True,
                       table_type='OTU table', process_obs_metadata='naive')

        obs = load_table(self.output_filepath)
        self.assertEqual(type(obs), Table)
        self.assertEqual(len(obs.ids()), 9)
        self.assertEqual(len(obs.ids(axis='observation')), 14)
        self.assertNotEqual(obs.metadata(), None)
        self.assertNotEqual(obs.metadata(axis='observation'), None)
        self.assertEqual(obs.metadata()[obs.index('p2', 'sample')],
                         {'foo': 'c;b;a'})
        self.assertEqual(obs.metadata()[obs.index('not16S.1', 'sample')],
                         {'foo': 'b;c;d'})
        self.assertEqual(obs.metadata(axis='observation')[
            obs.index('None11', 'observation')],
            {'taxonomy': 'Unclassified'})

        # With processing of metadata (currently only supports observation md).
        obs = self.cmd(table=self.classic_biom1,
                       output_filepath=self.output_filepath,
                       sample_metadata=self.sample_md1, table_type='OTU table',
                       process_obs_metadata='sc_separated', to_json=True)

        obs = load_table(self.output_filepath)
        self.assertEqual(type(obs), Table)
        self.assertEqual(len(obs.ids()), 9)
        self.assertEqual(len(obs.ids(axis='observation')), 14)
        self.assertNotEqual(obs.metadata(), None)
        self.assertNotEqual(obs.metadata(axis='observation'), None)
        self.assertEqual(obs.metadata()[obs.index('p2', 'sample')],
                         {'foo': 'c;b;a'})
        self.assertEqual(obs.metadata()[obs.index('not16S.1', 'sample')],
                         {'foo': 'b;c;d'})
        self.assertEqual(obs.metadata(axis='observation')[
            obs.index('None11', 'observation')],
            {'taxonomy': ['Unclassified']})

    def test_biom_to_classic1(self):
        """Correctly converts biom to classic."""
        self.cmd(table=self.biom_table1,
                 output_filepath=self.output_filepath,
                 to_tsv=True, header_key='taxonomy')

        self.assertEqual(load_table(self.output_filepath), self.classic_biom1)

    def test_biom_to_classic2(self):
        """Correctly converts biom to classic with metadata renaming."""
        self.cmd(table=self.biom_table1,
                 output_filepath=self.output_filepath, to_tsv=True,
                 header_key='taxonomy', output_metadata_id='foo')
        obs = load_table(self.output_filepath)
        self.assertTrue('foo' in obs.metadata(axis='observation')[0])

    def test_json_to_hdf5_collapsed_samples(self):
        """Correctly converts json to HDF5 changing the sample metadata"""
        with biom_open(self.json_collapsed_samples) as f:
            obs = self.cmd(table=parse_biom_table(f),
                           output_filepath=self.output_filepath, to_hdf5=True,
                           collapsed_samples=True)
        obs = load_table(self.output_filepath)
        exp = Table(np.array([[0., 1.], [6., 6.], [6., 1.],
                              [1., 4.], [0., 2.]]),
                    observation_ids=[
                        'GG_OTU_1',
                        'GG_OTU_2',
                        'GG_OTU_3',
                        'GG_OTU_4',
                        'GG_OTU_5',
                    ],
                    sample_ids=['skin', 'gut'],
                    observation_metadata=[
                        {'taxonomy': [
                            'k__Bacteria',
                            'p__Proteobacteria',
                            'c__Gammaproteobacteria',
                            'o__Enterobacteriales',
                            'f__Enterobacteriaceae',
                            'g__Escherichia',
                            's__',
                        ]},
                        {'taxonomy': [
                            'k__Bacteria',
                            'p__Cyanobacteria',
                            'c__Nostocophycideae',
                            'o__Nostocales',
                            'f__Nostocaceae',
                            'g__Dolichospermum',
                            's__',
                        ]},
                        {'taxonomy': [
                            'k__Archaea',
                            'p__Euryarchaeota',
                            'c__Methanomicrobia',
                            'o__Methanosarcinales',
                            'f__Methanosarcinaceae',
                            'g__Methanosarcina',
                            's__',
                        ]},
                        {'taxonomy': [
                            'k__Bacteria',
                            'p__Firmicutes',
                            'c__Clostridia',
                            'o__Halanaerobiales',
                            'f__Halanaerobiaceae',
                            'g__Halanaerobium',
                            's__Halanaerobiumsaccharolyticum',
                        ]},
                        {'taxonomy': [
                            'k__Bacteria',
                            'p__Proteobacteria',
                            'c__Gammaproteobacteria',
                            'o__Enterobacteriales',
                            'f__Enterobacteriaceae',
                            'g__Escherichia',
                            's__',
                        ]}],
                    sample_metadata=[
                        {'collapsed_ids': [
                            'Sample4',
                            'Sample5',
                            'Sample6',
                        ]},
                        {'collapsed_ids': [
                            'Sample1',
                            'Sample2',
                            'Sample3',
                        ]}
                    ],
                    type='OTU table')
        self.assertEqual(obs, exp)

    def test_json_to_hdf5_collapsed_metadata(self):
        """Correctly converts json to HDF5 changing the observation metadata"""
        with biom_open(self.json_collapsed_obs) as f:
            t = parse_biom_table(f)
            obs = self.cmd(table=t,
                           output_filepath=self.output_filepath, to_hdf5=True,
                           collapsed_observations=True)
        obs = load_table(self.output_filepath)
        exp = Table(np.array([[2., 1., 1., 0., 0., 1.],
                              [0., 0., 1., 4., 0., 2.],
                              [5., 1., 0., 2., 3., 1.],
                              [0., 1., 2., 0., 0., 0.]]),
                    observation_ids=[
                        'p__Firmicutes',
                        'p__Euryarchaeota',
                        'p__Cyanobacteria',
                        'p__Proteobacteria',
                    ],
                    sample_ids=[
                        'Sample1',
                        'Sample2',
                        'Sample3',
                        'Sample4',
                        'Sample5',
                        'Sample6',
                    ],
                    observation_metadata=[
                        {'collapsed_ids': ['GG_OTU_4']},
                        {'collapsed_ids': ['GG_OTU_3']},
                        {'collapsed_ids': ['GG_OTU_2']},
                        {'collapsed_ids': ['GG_OTU_1', 'GG_OTU_5']}],
                    sample_metadata=[
                        {'LinkerPrimerSequence': 'CATGCTGCCTCCCGTAGGAGT',
                         'BarcodeSequence': 'CGCTTATCGAGA',
                         'Description': 'human gut',
                         'BODY_SITE': 'gut'},
                        {'LinkerPrimerSequence': 'CATGCTGCCTCCCGTAGGAGT',
                         'BarcodeSequence': 'CATACCAGTAGC',
                         'Description': 'human gut',
                         'BODY_SITE': 'gut'},
                        {'LinkerPrimerSequence': 'CATGCTGCCTCCCGTAGGAGT',
                         'BarcodeSequence': 'CTCTCTACCTGT',
                         'Description': 'human gut',
                         'BODY_SITE': 'gut'},
                        {'LinkerPrimerSequence': 'CATGCTGCCTCCCGTAGGAGT',
                         'BarcodeSequence': 'CTCTCGGCCTGT',
                         'Description': 'human skin',
                         'BODY_SITE': 'skin'},
                        {'LinkerPrimerSequence': 'CATGCTGCCTCCCGTAGGAGT',
                         'BarcodeSequence': 'CTCTCTACCAAT',
                         'Description': 'human skin',
                         'BODY_SITE': 'skin'},
                        {'LinkerPrimerSequence': 'CATGCTGCCTCCCGTAGGAGT',
                         'BarcodeSequence': 'CTAACTACCAAT',
                         'Description': 'human skin',
                         'BODY_SITE': 'skin'}],
                    type='OTU table')

        self.assertEqual(obs, exp)


biom1 = """
{"id": "None",
 "format": "Biological Observation Matrix 1.0.0",
 "format_url": "http://biom-format.org",
 "type": "OTU table",
 "generated_by": "QIIME 1.6.0-dev",
 "date": "2013-02-09T09:30:11.550590",
 "matrix_type": "sparse",
 "matrix_element_type": "float",
 "shape": [14, 9],
 "data": [[0,0,20],[0,1,18],[0,2,18],[0,3,22],[0,4,4],[1,4,1],[2,0,1],
         [2,4,1],[2,5,1],[3,6,1],[4,4,1],[5,7,20],[6,4,1],[7,4,1],[7,5,1],
         [8,4,1],[8,6,2],[8,8,3],[9,7,2],[10,5,1],[11,4,9],[11,5,20],
         [11,6,1],[11,8,4],[12,4,3],[12,6,19],[12,8,15],[13,0,1],[13,1,4],
         [13,2,4]],
 "rows": [{"id": "295053",
           "metadata": {"taxonomy": ["k__Bacteria"]}},
          {"id": "42684", "metadata": {"taxonomy": ["k__Bacteria",
                                                    "p__Proteobacteria"]}},
          {"id": "None11", "metadata": {"taxonomy": ["Unclassified"]}},
          {"id": "None10", "metadata": {"taxonomy": ["Unclassified"]}},
          {"id": "None7", "metadata": {"taxonomy": ["Unclassified"]}},
          {"id": "None6", "metadata": {"taxonomy": ["Unclassified"]}},
          {"id": "None5", "metadata": {"taxonomy": ["k__Bacteria"]}},
          {"id": "None4", "metadata": {"taxonomy": ["Unclassified"]}},
          {"id": "None3", "metadata": {"taxonomy": ["k__Bacteria"]}},
          {"id": "None2", "metadata": {"taxonomy": ["k__Bacteria"]}},
          {"id": "None1", "metadata": {"taxonomy": ["Unclassified"]}},
          {"id": "879972", "metadata": {"taxonomy": ["k__Bacteria"]}},
          {"id": "None9", "metadata": {"taxonomy": ["Unclassified"]}},
          {"id": "None8", "metadata": {"taxonomy": ["k__Bacteria"]}}],
 "columns": [{"id": "f2", "metadata": null},
             {"id": "f1", "metadata": null},{"id": "f3", "metadata": null},
             {"id": "f4", "metadata": null},{"id": "p2", "metadata": null},
             {"id": "p1", "metadata": null},{"id": "t1", "metadata": null},
             {"id": "not16S.1", "metadata": null},
             {"id": "t2", "metadata": null}]
 }"""

classic1 = """# Constructed from biom file
#OTU ID\tf2\tf1\tf3\tf4\tp2\tp1\tt1\tnot16S.1\tt2\ttaxonomy
295053\t20.0\t18.0\t18.0\t22.0\t4.0\t0.0\t0.0\t0.0\t0.0\tk__Bacteria
42684\t0.0\t0.0\t0.0\t0.0\t1.0\t0.0\t0.0\t0.0\t0.0\tk__Bacteria; """ + \
    """p__Proteobacteria
None11\t1.0\t0.0\t0.0\t0.0\t1.0\t1.0\t0.0\t0.0\t0.0\tUnclassified
None10\t0.0\t0.0\t0.0\t0.0\t0.0\t0.0\t1.0\t0.0\t0.0\tUnclassified
None7\t0.0\t0.0\t0.0\t0.0\t1.0\t0.0\t0.0\t0.0\t0.0\tUnclassified
None6\t0.0\t0.0\t0.0\t0.0\t0.0\t0.0\t0.0\t20.0\t0.0\tUnclassified
None5\t0.0\t0.0\t0.0\t0.0\t1.0\t0.0\t0.0\t0.0\t0.0\tk__Bacteria
None4\t0.0\t0.0\t0.0\t0.0\t1.0\t1.0\t0.0\t0.0\t0.0\tUnclassified
None3\t0.0\t0.0\t0.0\t0.0\t1.0\t0.0\t2.0\t0.0\t3.0\tk__Bacteria
None2\t0.0\t0.0\t0.0\t0.0\t0.0\t0.0\t0.0\t2.0\t0.0\tk__Bacteria
None1\t0.0\t0.0\t0.0\t0.0\t0.0\t1.0\t0.0\t0.0\t0.0\tUnclassified
879972\t0.0\t0.0\t0.0\t0.0\t9.0\t20.0\t1.0\t0.0\t4.0\tk__Bacteria
None9\t0.0\t0.0\t0.0\t0.0\t3.0\t0.0\t19.0\t0.0\t15.0\tUnclassified
None8\t1.0\t4.0\t4.0\t0.0\t0.0\t0.0\t0.0\t0.0\t0.0\tk__Bacteria"""

sample_md1 = """#SampleID\tfoo
f4\ta;b;c
not16S.1\tb;c;d
f2\ta;c;d
f1\ta;b;c
p2\tc;b;a
f3\ta;b;c
t1\tt;b;c
p1\tp;b;c
t2\tt;2;z
"""


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# ----------------------------------------------------------------------------
# Copyright (c) 2023--, Panpiper development team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file COPYING.txt, distributed with this software.
# ----------------------------------------------------------------------------

import os
from os import path
from io import StringIO
import json
from unittest import TestCase
import sys
import numpy as np
import numpy.testing as npt
import pytest

sys.path.append(path.dirname(path.dirname(__file__)))
import main

__author__ = "Renee Oles"
__copyright__ = "Copyright 2023, Panpiper development team"
__credits__ = ["Renee Oles"]
__license__ = "BSD"
__maintainer__ = "Renee Oles"
__email__ = "roles@health.ucsd.edu"

class MainTest(TestCase):

    """Tests of parse functions"""
    def setUp(self):
        """define some top-level data"""
        self.outpath = outpath # set to test_data dir 
        self.fastq = fastq # test_data/fastq
        self.assembly1 = assembly1 # test_data/fasta1
        self.assembly2 = assembly2 # test_data/fasta2
        self.list1 = list1 # test_data/files.txt
        self.list2 = list2 # test_data/sample_list.txt
        self.reference = reference # test_data/9343.fna
        self.reference_empty = reference_empty # empty fasta file

    def test_unlock(self):
        exp = 0
        obs = ["-o", self.outpath, "--unlock"]
        sys.argv[1:] = obs
        self.assertEqual(main.cli(), exp)

    def test_assembly(self):
        exp = 0
        obs = ['--dry_run', '-o', self.outpath, '-w', 'assembly', '-q', self.fastq]
        self.assertEqual(main.cli(), exp)

    def test_quality(self):
        exp = 0
        obs = ['--dry_run', '-o', self.outpath, '-w', 'quality', '-a', self.assembly1, '-s', self.list1, '-r', self.reference]
        self.assertEqual(main.cli(), exp)

    def test_pangenome(self):
        exp = 0
        obs = ['--dry_run', '-o', self.outpath, '-w', 'pangenome', '-a', self.assembly2, '-s', self.list2, '-r', self.reference]
        self.assertEqual(main.cli(), exp)

    def test_wrong_assembly_quality(self):
        exp = 0
        obs = ['--dry_run', '-o', self.outpath, '-w', 'quality', '-a', self.assembly2, '-s', self.list1, '-r', self.reference]
        self.assertEqual(main.cli(), exp)

    def test_wrong_assembly_pangenome(self):
        exp = 0
        obs = ['--dry_run', '-o', self.outpath, '-w', 'pangenome', '-a', self.assembly1, '-s', self.list2, '-r', self.reference]
        self.assertEqual(main.cli(), exp)

    def test_no_outpath(self):
        exp = 0
        obs = ['--dry_run', '-w', 'assembly', '-q', self.fastq]
        self.assertEqual(main.cli(), exp)

    def test_incorrect_input(self):
        exp = 0
        obs = ['--dry_run', '-o', self.outpath, '-w', 'quality', '-q', self.fastq, '-s', self.list1, '-r', self.reference]
        self.assertEqual(main.cli(), exp)

    def test_no_reference(self):
        exp = 0
        obs = ['--dry_run', '-o', self.outpath, '-w', 'quality', '-a', self.assembly1, '-s', self.list1]
        self.assertEqual(main.cli(), exp)

    def test_empty_refernce(self):
        exp = 0
        obs = ['--dry_run', '-o', self.outpath, '-w', 'quality', '-a', self.assembly1, '-s', self.list1, '-r', self.reference_empty]
        self.assertEqual(main.cli(), exp)

outpath = "test_data"
fastq = "test_data/fastq"
assembly1 = "test_data/fasta1"
assembly2 = "test_data/fastaq"
list1 = "test_data/files.txt"
list2 = "test_data/sample_list.txt"
reference = "test_data/9343.fna"
reference_empty = "test_data/empty.fna"

run = MainTest()
run.setUp()
run.test_unlock()
run.test_assembly()
run.test_quality()
run.test_pangenome()
run.test_wrong_assembly_quality()
run.test_wrong_assembly_pangenome()
run.test_no_outpath()
run.test_incorrect_input()
run.test_no_reference()
run.test_empty_refernce()


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# File: main.py

# ----------------------------------------------------------------------------
# Panpiper: Pan-genome analysis pipeline
#
# Authors: Renee Oles
#
# Copyright (c) 2022--, Panpiper development team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file COPYING.txt, distributed with this software.
# ----------------------------------------------------------------------------

import argparse
import logging
import os
import sys
import pkg_resources
import importlib
from os.path import join, dirname, basename
from shutil import copyfile
from argparse import RawTextHelpFormatter
from panpiper.test import Test
from panpiper.workflow import Workflow

# Snakefiles
_ROOT = os.path.abspath(os.path.dirname(__file__))
WORKFLOW_ASSEMBLY = os.path.join(_ROOT, 'workflow', 'assembly.smk')
WORKFLOW_QUALITY = os.path.join(_ROOT, 'workflow', 'quality.smk')
WORKFLOW_PANGENOME = os.path.join(_ROOT, 'workflow', 'pangenome.smk')
WORKFLOW_PYSEER = os.path.join(_ROOT, 'workflow', 'pyseer.smk')
WORKFLOW_SCRIPTS = os.path.join(_ROOT, 'workflow', 'scripts')

def cli():
    ########## Arguments ##########
    ap = argparse.ArgumentParser(description='Package version ', add_help=False,
                                 allow_abbrev=False, formatter_class=RawTextHelpFormatter)

    # Required
    apr = ap.add_argument_group('main arguments')
    apr.add_argument('-q', '--fastq', help='Path to fastq file directory',
                     required=False, default="skip", type=str)
    apr.add_argument('-a', '--fasta', help='Path to fasta file directory',
                     required=False, default="skip", type=str)
    apr.add_argument('-s', '--sample_list', help='Line delimited file of samples passing quality control',
                     required=False, default="skip", type=str)
    apr.add_argument('-g', '--genes', help='File of gene presence/absence output from Panaroo/Roary',
                     required=False, default="skip", type=str)
    apr.add_argument('-p', '--structure', help='File of structure presence/absence output from Panaroo/Roary',
                     required=False, default="skip", type=str)
    apr.add_argument('-u', '--unitig', help='File of unitig list',
                     required=False, default="skip", type=str)
    apr.add_argument('-t', '--tree', help='Newick tree of all samples from IQTree',
                     required=False, default="skip", type=str)
    apr.add_argument('-o', '--output',
                     help='Prefix for output directory', required=True)
    apr.add_argument('-r', '--reference', help='Reference fasta file, includes path',
                     required=False, default="skip", type=str)
    apr.add_argument('-w', '--workflow', help='Choose which workflow to run [%(default)s]',
                     default='assembly', type=str, choices=['assembly', 'quality', 'pangenome', 'pyseer'])
    apr.add_argument(
        '-f', '--pheno', help='The filename and full path of the phenotype file', default="skip", type=str)

    # Cluster arguments
    apc = ap.add_argument_group('compute cluster arguments')
    apc.add_argument('--cluster', help='Run with cluster', action='store_true')
    apc.add_argument('--profile',help='Profile folder contain contig file [%(default)s]', default=None, type=str)

    # Optional
    apo = ap.add_argument_group('optional arguments')
    apo.add_argument("-h", "--help", action="help",help="show this help message and exit")
    apo.add_argument('--log_lvl', help='Logging level [%(default)s].',
                     default='INFO', type=str, choices=['DEBUG', 'INFO', 'WARNING', 'ERROR'])
    apo.add_argument('--only_conda', help='Only install conda environments, then exit', action='store_true')
    apo.add_argument(
        '--snake', help='Only run specific snakemake command. For debug purposes', type=str)
    apo.add_argument(
        '--dry_run', help='Only run specific snakemake command. For debug purposes', action='store_true')
    apo.add_argument(
        '--unlock', help='Unlock snakemake directory in case of unexpected exists, then exit', action='store_true')

    # Parameters
    # Defaults set for Bacteroides fragilis
    app = ap.add_argument_group('parameters')
    apo.add_argument(
        '--threads', help='Specify the number of threads', default="8", type=str)
    app.add_argument(
        '--ani_cutoff', help='Average percent identity to reference cutoff', default=95, type=float)
    app.add_argument('--contig_number',
                     help='Max number of contigs', default=1000, type=int)
    app.add_argument('--n50', help='N50 cutoff', default=5000, type=int)
    app.add_argument('--eggnog_dir', help='Path to the eggnog database directory',
                     default=os.path.join(_ROOT, "databases", "eggnog"), type=str)
    app.add_argument('--bakta_dir', help='Path to the bakta database directory',
                     default=os.path.join(_ROOT, "databases", "bakta"), type=str)
    app.add_argument('--checkm_dir', help='Path to the checkm database directory',
                     default=os.path.join(_ROOT, "databases", "checkm2"), type=str)
    app.add_argument(
        '--pheno_column', help='The column in the phenotype file to use for the association study', type=str)
    app.add_argument(
        '--max_jobs', help='Maximum number of cluster jobs [%(default)s]', default=40, type=int)
    app.add_argument('--scripts_dir', help='Directory of scripts in package, do not change unless you have modified the scripts',
        type=str, default=WORKFLOW_SCRIPTS)

    ########## Workflow ##########
    master = Test(ap)

    # Create the directory if it doesn't exist
    if not os.path.exists(master.eggnog_dir):
        os.makedirs(master.eggnog_dir)

    # Create the directory if it doesn't exist
    if not os.path.exists(master.bakta_dir):
        os.makedirs(master.bakta_dir)

    wf = Workflow(master)

    # If only 1 snakemake command should be run
    if master.snake:
        logging.info('Only running '+master.snake+' snakefile')
        wf.run(snakefile=globals()['WORKFLOW_'+master.snake])

    # If workflow is set to Assembly
    elif (master.workflow == "assembly"):
        # Assembly
        logging.info('Assemble fastq')
        wf.run(snakefile=WORKFLOW_ASSEMBLY)

    # If workflow is set to Quality
    elif (master.workflow == "quality"):
        # Quality control
        logging.info(
            'Run quality control for assembly quality and taxonomy verification')
        logging.info('The parameters used in this study are- , Reference: ' + master.reference +
                     ', ANI Cutoff: ' + str(master.ani_cutoff) + ', Contig Number: ' + str(master.contig_number) + ', N50: ' + str(master.n50))
        wf.run(snakefile=WORKFLOW_QUALITY)

    # If workflow is set to Pangenome
    elif (master.workflow == "pangenome"):
        logging.info('Run pangenome creation and analysis')
        logging.info('The parameters used in this study are- Eggnog Database Directory: ' + master.eggnog_dir +
                     ', Bakta Database Directory: ' + master.bakta_dir)
        wf.run(snakefile=WORKFLOW_PANGENOME)

    # If workflow is set to Pyseer
    elif (master.workflow == "pyseer"):
        logging.info('Run genome-wide association study')
        logging.info(
            'The parameters used in this study are- Phenotype column: ' + str(master.pheno_column))
        wf.run(snakefile=WORKFLOW_PYSEER)


if __name__ == '__main__':
    cli()



# ----------------------------------------------------------------------------
# Copyright (c) 2022--, Panpiper development team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file COPYING.txt, distributed with this software.
# ----------------------------------------------------------------------------

import os
import logging
import sys
import pkg_resources
import re
from Bio import SeqIO
import gzip


class Test(object):
    """
    This class evaluates all user input to prepare for snakemake run.
    """

    def __init__(self, ap):

        args = ap.parse_args()

        for k, v in args.__dict__.items():
            setattr(self, k, v)

        # Logger
        logging.basicConfig(format='\033[36m'+'[%(asctime)s] %(levelname)s:' +
                            '\033[0m'+' %(message)s', datefmt='%Y-%m-%d %H:%M:%S', level=self.log_lvl)
        logging.info('Running Panpiper version {}'.format(
            pkg_resources.require("panpiper")[0].version))

        # Force consistency
        self.output = os.path.join(self.output, '')

        # Check cluster info input
        # TODO: update to check cluster information
        if self.cluster:
            if self.profile is None:
                logging.error('A profile folder must be specified')
                sys.exit()
            else: 
                profile = os.path.join(self.profile, "config.yaml")
                if not os.path.isfile(profile):
                    logging.error('config.yaml file not found in profile folder')
                    sys.exit()

        # Check input, params, and output
        self.check_out()
        if not any([self.unlock, self.only_conda, self.snake is not None]):
            if(self.workflow == "assembly"):
                self.check_fastq()
            elif(self.workflow == "quality"):
                self.check_reference()
                self.check_list()
            elif(self.workflow == "pangenome"):
                self.check_list()
                self.check_reference()
            elif(self.workflow == "pyseer"):
                self.check_pyseer()

        self.write_params(args)

    def check_out(self):
        """
        Tries to make output directory

        Raises
        ------
        Warning
            'Warning: Output directory already exists'
        """
        try:
            os.makedirs(self.output)
        except FileExistsError:
            logging.warning('Output directory '+self.output+' already exists')

    def check_fastq(self):
        """
        Tries to find input fastq files

        Raises
        ------
        Exception
            'Exception: Fastq files not found'
        """
        logging.debug('Checking input directory for fastq files')
        for file in os.listdir(self.fastq):
            if file.endswith(".fastq"):
                logging.debug('Fastq files found')
                return()

        logging.error('No fastq files found in directory')
        sys.exit()

    def check_reference(self):
        """
        Tries to open reference fasta file 

        Raises
        ------
        Exception
            'Exception: Reference file not found'

        ValueError
            'ValueError: Reference fasta file not in fasta format'
        """
        logging.debug('Checking reference fasta file')

        try:
            with open(self.reference, "r") as handle:
                fasta = SeqIO.parse(handle, "fasta")
        except Exception:
            logging.error('Reference file not found')
            sys.exit()

        if not (fasta):
            logging.error('Reference fasta file not in fasta format')
            sys.exit()

    def check_list(self):
        """
        Checks sample list file for fasta file list 
        """
        logging.debug('Checking sample list file')

        with open(self.sample_list, 'r') as fh:
            for line in fh:
                if not os.path.join(self.fasta, line, ".fa"):
                    logging.error(
                        'Sample names not present in fasta directory')
                    sys.exit()

    def check_pyseer(self):
        # genes : gene_presence_absence.Rtab check if exists
        if not os.path.isfile(self.genes):
            logging.error('Gene presence/absence file does not exist')
            sys.exit()

        # structure : struct_presence_absence.Rtab check if exists
        if not os.path.isfile(self.structure):
            logging.error('Structure presence/absence file does not exist')
            sys.exit()

        # unitig : unitig.pyseer.gz check that it's zipped
        with gzip.open(self.unitig, 'r') as fh:
            try:
                fh.read(1)
            except gzip.BadGzipFile:
                print('Unitig is a bad gzip file')
                sys.exit()

        # tree : iqtree.nwk check if in newick format
        if not os.path.isfile(self.tree):
            logging.error('Tree file does not exist')
            sys.exit()

        # reference : check if file in text can be found


        # pheno : check if file has sample name as first column

    def write_params(self, args):
        """
        Add parameters to params to snakemake file 
        """

        pars = vars(args)
        self.params = self.output+'parameters.tab'
        fh = open(self.params, 'w')
        for k, v in pars.items():
            fh.write('{}\t{}\n'.format(k, v))
        if not self.reference == "skip":
            ref = os.path.basename(self.reference)
            ref = os.path.splitext(ref)[0]
            fh.write('ref\t{}\n'.format(ref))
        fh.close()



#!/usr/bin/env python

# ----------------------------------------------------------------------------
# Copyright (c) 2022--, Panpiper development team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file COPYING.txt, distributed with this software.
# ----------------------------------------------------------------------------

import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="panpiper", 
    version="1.0.2",
    license='BSD-3-Clause',
    author="Renee Oles",
    author_email="roles@health.ucsd.edu",
    description="Panpiper: snakemake workflow for bacterial isolate analysis",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/rolesucsd/Panpiper",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Development Status :: 3 - Alpha"],
    python_requires='>=3.5',
    install_requires=[
        "setuptools",
        "matplotlib",
        "seaborn",
        "scikit-learn",
        "umap-learn",
        "biopython"],
    entry_points = {
        'console_scripts': ['panpiper = panpiper.main:cli']
    },
    include_package_data=True,
    zip_safe=False,
    package_data={
    'panpiper': [
        'workflow/*',  # This includes files directly under workflow
        'workflow/envs/*',  # This includes files in the envs subdirectory
        'workflow/scripts/*'  # This includes files in the scripts subdirectory
        'databases/*'  # This includes files in the scripts subdirectory
        'databases/checkm2/*'  # This includes files in the scripts subdirectory
        ]
    }
)
